@@ -69,6 +69,8 @@
 #include "icons.h"
 #include "threads.h"
 
+#include "ext_string.h"
+
 #ifdef WIN32
 #  include "flmsgrc.h"
 #  include "compat.h"
@@ -97,6 +99,7 @@ static string option_str   = "<OPTION";
 static string textarea_str = "<TEXTAREA";
 static string textend_str  = "</TEXTAREA";
 static string checked      = "CHECKED";
+static string checkedeq    = "CHECKED=\"CHECKED\"";
 static string selected     = "SELECTED";
 
 static string value_str    = "VALUE=\"";
@@ -140,111 +143,75 @@ std::vector<NAME_VALUE> name_values;
 string html_form;
 string edit_txt;
 
-void convert_case(string &s)
+void escape(string &s)
 {
 	size_t p;
-	// convert FORM
-	p = s.find("<form", 0);
-	while (p != string::npos) {
-		s.replace(p, 5, "<FORM");
-		p = s.find("<form", p+5);
+	p = 0;
+	while ((p = s.find("\r", p)) != string::npos)
+		s.erase(p,1);
+	p = 0;
+	while ((p = s.find("\n", p)) != string::npos) {
+		s.replace(p, 1, "\\n");
+		p += 2;
 	}
-	p = s.find("</form", 0);
-	while (p != string::npos) {
-		s.replace(p, 6, "</FORM");
-		p = s.find("</form", p+6);
+	p = 0;
+	while ((p = s.find("\"", p)) != string::npos) {
+		if (p == 0 || s[p-1] != '\\') {
+			s.replace(p, 1, "\\\"");
+			p += 2;
+		} else p++;
 	}
-	// convert INPUT
-	p = s.find("<input", 0);
-	while (p != string::npos) {
-		s.replace(p, 6, "<INPUT");
-		p = s.find("<input", p+6);
+}
+
+void unescape(string &s)
+{
+	size_t p;
+	p = 0;
+	while ((p = s.find("\\n", p)) != string::npos) {
+		s.replace(p,2,"\n");
+		p++;
 	}
-	// convert TEXTAREA
-	p = s.find("<textarea", 0);
-	while (p != string::npos) {
-		s.replace(p, 9, "<TEXTAREA");
-		p = s.find("<textarea", p+9);
+	p = 0;
+	while ((p = s.find("\\\"", p)) != string::npos) {
+		s.replace(p,2,"\"");
+		p++;
 	}
-	// convert VALUE
-	p = s.find("value=\"", 0);
-	while (p != string::npos) {
-		s.replace(p, 7, "VALUE=\"");
-		p = s.find("value=\"", p+7);
-	}
-	// convert NAME
-	p = s.find("name=\"", 0);
-	while (p != string::npos) {
-		s.replace(p, 6, "NAME=\"");
-		p = s.find("name=\"", p+6);
-	}
-	// convert VALUE
-	p = s.find("value=\"", 0);
-	while (p != string::npos) {
-		s.replace(p, 7, "VALUE=\"");
-		p = s.find("value=\"", p+7);
-	}
-	// convert TYPE
-	p = s.find("type=\"", 0);
-	while (p != string::npos) {
-		s.replace(p, 6, "TYPE=\"");
-		p = s.find("type=\"", p+6);
-	}
-	// convert ="TEXT
-	p = s.find("=\"text", 0);
-	while (p != string::npos) {
-		s.replace(p, 6, "=\"TEXT");
-		p = s.find("=\"text", p+6);
-	}
-	// convert ="RADIO
-	p = s.find("=\"radio", 0);
-	while (p != string::npos) {
-		s.replace(p, 7, "=\"RADIO");
-		p = s.find("=\"radio", p+7);
-	}
-	// convert ="CHECKBOX
-	p = s.find("=\"checkbox", 0);
-	while (p != string::npos) {
-		s.replace(p, 10, "=\"CHECKBOX");
-		p = s.find("=\"checkbox", p+10);
-	}
-	// convert checked
-	p = s.find("checked", 0);
-	while (p != string::npos) {
-		s.replace(p, 7, "CHECKED");
-		p = s.find("checked", p+7);
-	}
-	p = s.find("selected", 0);
-	while (p != string::npos) {
-		s.replace(p, 8, "SELECTED");
-		p = s.find("selected", p+8);
-	}
-	p = s.find("<select", 0);
-	while (p != string::npos) {
-		s.replace(p, 7, "<SELECT");
-		p = s.find("<select", p+7);
-	}
-	p = s.find("</select", 0);
-	while (p != string::npos) {
-		s.replace(p, 8, "</SELECT");
-		p = s.find("</select", p+8);
-	}
-	p = s.find("<option", 0);
-	while (p != string::npos) {
-		s.replace(p, 6, "<OPTION");
-		p = s.find("<option", p+6);
-	}
-	p = s.find("TYPE=\"password", 0);
-	while (p != string::npos) {
-		s.replace(p, 14, "TYPE=\"PASSWORD");
-		p = s.find("TYPE=\"password", p+14);
-	}
-//printf("%s\n", s.c_str());
+}
+
+int convert_case(string &s)
+{
+	extstring mystring;
+	mystring.assign(s);
+	if (mystring.ufind("<FORM", 0) == string::npos ||
+		mystring.ufind("</FORM", 0) == string::npos )
+		return 1;
+	mystring.ureplace("<FORM");
+	mystring.ureplace("</FORM");
+	mystring.ureplace("<INPUT");
+	mystring.ureplace("<TEXTAREA");
+	mystring.ureplace("</TEXTAREA");
+	mystring.ureplace("VALUE=\"");
+	mystring.ureplace("NAME=\"");
+	mystring.ureplace("TYPE=\"");
+	mystring.ureplace("=\"TEXT\"");
+	mystring.ureplace("=\"RADIO\"");
+	mystring.ureplace("=\"CHECKBOX\"");
+	mystring.ureplace("=\"PASSWORD\"");
+	mystring.ureplace("SELECTED");
+	mystring.ureplace("CHECKED");
+	mystring.ureplace("<SELECT");
+	mystring.ureplace("</SELECT");
+	mystring.ureplace("<OPTION");
+	s.assign(mystring);
+	return 0;
 }
 
 // this function may be called by the main or the web_server thread
+// reads the values associated with each form type and
+// clears the html form of those value.
 void extract_fields()
 {
+//printf("custom select %d\n", custom_select);
 	if (custom_select < 0) return;
 	string fname = CUSTOM_dir;
 	{
@@ -260,11 +227,17 @@ void extract_fields()
 	while ((c = fgetc(html_file)) != EOF) html_form += c;
 	fclose(html_file);
 
-	convert_case(html_form);
+	if (convert_case(html_form)) {
+		html_form.clear();
+		custom_select = 0;
+		edit_txt.assign("INVALID HTML FORM DOCUMENT");
+		return;
+	}
 
 	size_t ptype, pstart, pend, pname, pvalue, p1, p2, p3;
 	string field_name;
 	string field_value;
+	string val;
 
 	name_values.clear();
 
@@ -292,7 +265,12 @@ void extract_fields()
 					if (pvalue == string::npos || pvalue > pend) break;
 					pvalue += value_str.length();
 					p2 = html_form.find("\"", pvalue);
-					field_value = html_form.substr(pvalue, p2 - pvalue);
+					val = html_form.substr(pvalue, p2 - pvalue);
+					p3 = val.find("&quot;");
+					while (p3 != string::npos)
+						val.replace(p3, 6, "\"");
+					escape(val);
+					field_value = val;
 					break;
 				case T_RADIO:
 					pvalue = html_form.find(value_str, pstart);
@@ -302,10 +280,20 @@ void extract_fields()
 					if (p1 < pend) {
 						field_name.append(".")
 								  .append(html_form.substr(pvalue, p1 - pvalue));
-						if (html_form.substr(pstart, pend-pstart).find(checked) != string::npos) {
+						field_value.clear();
+						p2 = html_form.find(checkedeq, pstart);
+						if (p2 < pend) {
 							field_value = checked;
-						} else
-							field_value.clear();
+							html_form.erase(p2 - 1, checkedeq.length() + 1);
+							pend = html_form.find(">", pstart);
+						} else {
+							p2 = html_form.find(checked, pstart);
+							if (p2 < pend) {
+								field_value = checked;
+								html_form.erase(p2 - 1, checked.length() + 1);
+								pend = html_form.find(">", pstart);
+							}
+						}
 					}
 					break;
 				case T_CHECKBOX:
@@ -318,19 +306,39 @@ void extract_fields()
 									  .append(html_form.substr(pvalue, p1 - pvalue));
 						}
 					}
-					if (html_form.substr(pstart, pend-pstart).find(checked) != string::npos ||
-						html_form.substr(pstart, pend-pstart).find("ON") != string::npos) {
+					field_value.clear();
+					p2 = html_form.find(checkedeq, pstart);
+					if (p2 < pend) {
 						field_value = "ON";
-					} else
-						field_value.clear();
+						html_form.erase(p2 - 1, checkedeq.length() + 1);
+						pend = html_form.find(">", pstart);
+					} else {
+						p2 = html_form.find(checked, pstart);
+						if (p2 < pend) {
+							field_value = "ON";
+							html_form.erase(p2 - 1, checked.length() + 1);
+							pend = html_form.find(">", pstart);
+						} else {
+							p2 = html_form.find(" ON", pstart);
+							if (p2 < pend) {
+								field_value = "ON";
+								html_form.erase(p2, 3);
+								pend = html_form.find(">", pstart);
+							}
+						}
+					}
 					break;
-				case T_AREA:
+				case T_AREA: //extract
 					pvalue = pend + 1;
 					p1 = html_form.find(textend_str, pvalue);
 					if (p1 == string::npos) break;
-					field_value.assign("\"")
-							   .append(html_form.substr(pvalue, p1 - pvalue))
-							   .append("\"");
+					if (p1 > pvalue)
+						val = html_form.substr(pvalue, p1 - pvalue);
+					else
+						val.clear();
+					html_form.erase(pvalue, val.length());
+					escape(val);
+					field_value.assign(val);
 					break;
 				case T_SELECT:
 					p3 = html_form.find(end_sel_str, pstart);
@@ -349,15 +357,6 @@ void extract_fields()
 				default:
 					break;
 			}
-/*
-printf("%s : %s : %s\n",
-input_types[i].id == T_TEXT ? "TEXT" :
-input_types[i].id == T_RADIO ? "RADIO" :
-input_types[i].id == T_CHECKBOX ? "CHECKBOX" : 
-input_types[i].id == T_SELECT ? "SELECT" :
-input_types[i].id == T_PASSWORD ? "PASSWORD" :"AREA",
-field_name.c_str(), field_value.c_str() );
-*/
 			edit_txt.append(field_name).append(",");
 			edit_txt.append(field_value).append("\n");
 			name_values.push_back(NAME_VALUE(input_types[i].id, field_name, field_value));
@@ -376,7 +375,10 @@ void refresh_txt_custom_msg(void *)
 // called by web server thread
 void get_html_vars(struct mg_connection *conn)
 {
-	char buff[500];
+static char buff[5000];
+
+	memset(buff, 0, sizeof(buff));
+
 	size_t p;
 	if (custom_select < 0) return;
 
@@ -384,7 +386,10 @@ void get_html_vars(struct mg_connection *conn)
 			.append(custom_pairs[custom_select].file_name)
 			.append("\n");
 
-	string field, line;
+	string field, line, val;
+	for (size_t n = 0; n < name_values.size(); n++)
+		name_values[n].value.clear();
+
 	for (size_t n = 0; n < name_values.size(); n++) {
 		field = name_values[n].name;
 		if (name_values[n].id == T_CHECKBOX ||
@@ -392,15 +397,13 @@ void get_html_vars(struct mg_connection *conn)
 			if ((p = field.find(".")) != string::npos)
 				field.erase(p);
 		}
+		memset(buff, 0, sizeof(buff));
 		mg_get_var(conn, field.c_str(), buff, sizeof(buff));
-
 		switch (name_values[n].id) {
 			case T_RADIO :
 				p = name_values[n].name.find(".");
 				if (name_values[n].name.substr(p+1) == buff)
 					name_values[n].value = checked;
-				else
-					name_values[n].value.clear();
 				line.assign(name_values[n].name)
 					.append(",")
 					.append(name_values[n].value);
@@ -408,23 +411,15 @@ void get_html_vars(struct mg_connection *conn)
 			case T_CHECKBOX :
 				if (strstr(buff, "on") == buff || strstr(buff, "ON") == buff)
 					name_values[n].value = "ON";
-				else
-					name_values[n].value.clear();
 				line.assign(name_values[n].name)
 					.append(",")
 					.append(name_values[n].value);
 				break;
-			case T_AREA:
-				name_values[n].value = buff;
-				p = name_values[n].value.find("\r\n");
-				while (p != string::npos) {
-					name_values[n].value.erase(p,1);
-					p = name_values[n].value.find("\r\n");
-				}
-				line.assign(name_values[n].name)
-					.append(",").append("\"")
-					.append(name_values[n].value)
-					.append("\"");
+			case T_AREA://get html vars
+				val = buff;
+				escape(val);
+				name_values[n].value = val;
+				line.assign(name_values[n].name).append(",").append(val);
 				break;
 			case T_SELECT:
 			default :  // T_TEXT, T_PASSWORD, T_SELECT
@@ -438,21 +433,29 @@ void get_html_vars(struct mg_connection *conn)
 	Fl::awake(refresh_txt_custom_msg);
 }
 
+// modify the html form with the value strings
+
 void assign_values(string &html)
 {
 	string nm, val, s1, s2, temp;
-	size_t p, p0, p1, p2, p3;
+	size_t p, p0, p1, p2, p3, pbeg, pend, pval, pnm;
 
 	for (size_t n = 0; n < name_values.size(); n++) {
 		switch (name_values[n].id) {
 			case T_TEXT : case T_PASSWORD :
 				nm.assign("NAME=\"").append(name_values[n].name).append("\"");
-				p0 = html.find(nm);
-				if (p0 != string::npos) {
-					p0 += nm.length();
-					p1 = html.find(value_str, p0);
-					p2 = html.find(">", p0);
+				pnm = html.find(nm);
+				if (pnm != string::npos) {
+					pnm += nm.length();
+					p1 = html.find(value_str, pnm);
+					p2 = html.find(">", pnm);
 					val = name_values[n].value;
+					unescape(val);
+					p3 = val.find("\"");
+					while (p3 != string::npos) {
+						val.replace(p3,1, "&quot;");
+						p3 = val.find("\"");
+					}
 					if (p1 < p2) {
 						p1 += value_str.length();
 						p2 = html.find("\"", p1);
@@ -470,57 +473,57 @@ void assign_values(string &html)
 				s2.assign(temp.substr(p+1));
 				nm.assign(name_str).append(s1).append("\"");
 				val.assign(value_str).append(s2).append("\"");
-				p0 = html.find(nm);
-				while (p0 != string::npos) {
-					p1 = html.find(val, p0);
-					p2 = html.find(">", p0);
-					if (p1 == string::npos || p1 > p2) {
-						p0 = html.find(nm, p2);
+				pnm = html.find(nm);
+				while (pnm != string::npos) {
+					pbeg = html.rfind("<", pnm); // beginning of tag specifier
+					pend = html.find(">", pbeg); // end of tag specifier
+					pval = html.find(val, pbeg);
+					if (pval == string::npos || pval > pend) {
+						pnm = html.find(nm, pend);
 						continue;
 					}
 				// found name and value pair
-					p1 += val.length();
-					if (name_values[n].value == checked) {
-						html.replace(p1, p2 - p1, string(" ").append(checked));
-					} else if (p2 - p1 > 0) {
-						html.erase(p1, p2 - p1);
-					}
-					p2 = html.find(">", p0);
-
-					p0 = html.find(nm, p2);
+					if (name_values[n].value == checked)
+						html.insert(pend, string(" ").append(checked));
+					pend = html.find(">", pbeg);
+					pnm = html.find(nm, pend);
 				}
 				break;
 			case T_CHECKBOX :
 				nm.assign("NAME=\"").append(name_values[n].name).append("\"");
-				p = html.find(nm);
-				if (p != string::npos) {
-					p1 = p + nm.length();
-					p2 = html.find(">", p1);
+				pnm = html.find(nm);
+				if (pnm != string::npos) {
+					pbeg = html.rfind("<", pnm);
+					pend = html.find(">", pbeg);
 					if (name_values[n].value == "ON") {
-						html.replace(p1, p2 - p1, string(" ").append("CHECKED"));
-					} else {
-						html.replace(p1, p2 - p1, "");
+						html.insert(pend, string(" ").append(checked));
+						pend = html.find(">", pbeg);
 					}
 				}
 				break;
-			case T_AREA :
+			case T_AREA : //assign values
 				nm.assign("NAME=\"").append(name_values[n].name).append("\"");
-				p = html.find(nm);
-				if (p != string::npos) {
-					p1 = html.find("</TEXTAREA", p);
-					p0 = html.rfind(">", p1);
-					string val = name_values[n].value;
-					if (val[0] == '"') val.erase(0,1);
-					if (val[val.length() -1] == '"') val.erase(val.length()-1, 1);
-					html.replace(p0+1, p1-p0-1, val);
+				pnm = html.find(nm);
+				if (pnm != string::npos) {
+					p1 = html.find(textend_str, pnm);
+					if (p1 == string::npos)
+						break;
+					p0 = html.rfind(">", p1) + 1;
+					val = name_values[n].value;
+					unescape(val);
+					if (p0 < p1) {
+						html.replace(p0, p1 - p0, val);
+					} else {
+						html.insert(p1, val);
+					}
 				}
 				break;
 			case T_SELECT :
 				nm.assign("NAME=\"").append(name_values[n].name).append("\"");
-				p = html.find(nm);
-				if (p != string::npos) {
-					p2 = html.find("</SELECT", p);
-					p0 = html.find(value_str, p);
+				pnm = html.find(nm);
+				if (pnm != string::npos) {
+					p2 = html.find("</SELECT", pnm);
+					p0 = html.find(value_str, pnm);
 					while(p0 != string::npos && p0 < p2) {
 						p0 += value_str.length();
 						p1 = html.find("\"", p0);
@@ -551,9 +554,7 @@ void custom_editor(struct mg_connection *conn)
 	html_edit.replace(p, 6, action_str);
 	p = html_edit.find("</FORM>");
 	html_edit.replace(p, 7, submit_str);
-
 	assign_values(html_edit);
-
 	mg_send_data(conn, html_edit.c_str(), html_edit.length());
 }
 
@@ -651,6 +652,7 @@ bool using_custom_template = false;
 void clear_customfields()
 {
 	custom_field.clear();
+	extract_fields();
 }
 
 bool check_customfields()
@@ -667,33 +669,51 @@ void clear_custom_form()
 {
 	clear_customfields();
 	txt_custom_msg->clear();
+	txt_custom_msg->add(edit_txt.c_str());
 }
 
 void text_to_pairs()
 {
-	size_t p, p1, pquote;
+	string val;
+	size_t p, p1;
 	for (size_t n = 0; n < name_values.size(); n++) {
 		p = edit_txt.find(name_values[n].name);
 		if (p != string::npos) {
 			p += name_values[n].name.length() + 1;
-			pquote = edit_txt.find("\"", p);
 			p1 = edit_txt.find("\n", p);
-			if (pquote < p1) {
-				p = pquote + 1;
-				p1 = edit_txt.find("\"", p + 1);
-			}
-			name_values[n].value = edit_txt.substr(p, p1 - p);
+			val = edit_txt.substr(p, p1 - p);
+			if (!val.empty() &&
+				(name_values[n].id == T_AREA || name_values[n].id == T_TEXT))
+				escape(val);
+			name_values[n].value = val;
 		}
+//printf("name %s, value %s\n", name_values[n].name.c_str(), name_values[n].value.c_str());
 	}
 }
 
+void pairs_to_text()
+{
+	edit_txt.clear();
+	edit_txt.assign("CUSTOM_FORM,")
+			.append(custom_pairs[custom_select].file_name)
+			.append("\n");
+	for (size_t n = 0; n < name_values.size(); n++)
+		edit_txt.append(name_values[n].name)
+				.append(",")
+				.append(name_values[n].value)
+				.append("\n");
+}
+
 void update_customform()
 {
 	extract_fields();
 	edit_txt = custom_field;
+	text_to_pairs();
+//printf("B\n%s\n", edit_txt.c_str());
+	pairs_to_text();
+//printf("C\n%s\n", edit_txt.c_str());
 	txt_custom_msg->clear();
 	txt_custom_msg->add(edit_txt.c_str());
-	text_to_pairs();
 }
 
 void read_custombuffer(string data)
@@ -714,6 +734,7 @@ void read_custombuffer(string data)
 	string fname = custom_field.substr(p0, p1-p0);
 	string html_fname = CUSTOM_dir;
 	html_fname.append(fname);
+//printf("form file: %s\n", html_fname.c_str());
 
 	{ // treat this block as a critical section
 		guard_lock web_lock(&mutex_web_server);
@@ -1102,6 +1123,7 @@ void custom_set_fname(const char *fn)
 
 void cb_custom_html()
 {
+	return;
 	if (custom_field.find("CUSTOM_FORM") == 0) {
 		size_t plf = custom_field.find("\n");
 		if (plf != string::npos) {
@@ -1124,6 +1146,7 @@ void cb_custom_html()
 
 void cb_custom_textout()
 {
+	return;
 	if (custom_field.find("CUSTOM_FORM") == 0) {
 		size_t plf = custom_field.find("\n");
 		if (plf != string::npos) {
@@ -1137,3 +1160,117 @@ void cb_custom_textout()
 		}
 	}
 }
+
+//==============================================================================
+// Support for transfering / receiving custom FORM, html files
+//==============================================================================
+
+string transfer_custom_buffer;
+string def_custom_transfer_filename;
+
+string def_custom_rx_filename;
+string receive_custom_buffer;
+
+void read_custom_transfer_buffer(string data)
+{
+	const char *xfrstr = "<html_form>\n";
+	size_t p1 = data.find(xfrstr);
+	if (p1 != string::npos) p1 += strlen(xfrstr);
+	data.erase(0, p1);
+	receive_custom_buffer = data;
+	read_header(data);
+}
+
+void cb_custom_form_wrap_import(string wrapfilename, string inpbuffer)
+{
+	read_custom_transfer_buffer(inpbuffer);
+
+	def_custom_rx_filename = CUSTOM_dir;
+	def_custom_rx_filename.append(wrapfilename);
+	txt_rcvd_custom_html_filename->value(def_custom_rx_filename.c_str());
+	btn_save_custom_html_file->color(FL_RED);
+}
+
+int eval_transfer_custom_form_fsize()
+{
+	if (transfer_custom_buffer.empty()) return 0;
+
+	Ccrc16 chksum;
+
+	evalstr.assign("[WRAP:beg][WRAP:lf][WRAP:fn ");
+	evalstr.append(fl_filename_name(def_custom_transfer_filename.c_str())).append("]");
+	evalstr.append(header("<html_form>"));
+
+	string outbuf(transfer_custom_buffer);
+	if (outbuf.empty()) return 0;
+
+	compress_maybe( outbuf, false );
+
+	evalstr.append( outbuf );
+	string ck = chksum.scrc16(evalstr);
+
+	evalstr.append("[WRAP:chksum ").append(ck).append("][WRAP:end]");
+
+	return evalstr.length();
+}
+
+void load_custom_html_file()
+{
+	string fname = CUSTOM_dir;
+	fname.append(def_custom_transfer_filename);
+	transfer_custom_buffer.clear();
+
+	FILE *dfile = fopen(fname.c_str(), "rb");
+	if (!dfile) {
+		show_filename("ERROR");
+		transfer_custom_buffer.clear();
+		return;
+	}
+	fseek(dfile, 0, SEEK_END);
+	size_t fsize = ftell(dfile);
+	if (fsize <= 0) return;
+	fseek(dfile, 0, SEEK_SET);
+	transfer_custom_buffer.resize(fsize);
+	size_t r = fread((void *)transfer_custom_buffer.c_str(), 1, fsize, dfile);
+	fclose(dfile);
+	if (r != fsize) {
+		show_filename("ERROR");
+		transfer_custom_buffer.clear();
+		return;
+	}
+	estimate();
+}
+
+void cb_transfer_custom_html()
+{
+	if (transfer_custom_buffer.empty()) return;
+
+	update_header(FROM);
+	string fbuff(header("<html_form>"));
+	string outbuf(transfer_custom_buffer);
+	compress_maybe(outbuf, false);
+	fbuff.append(outbuf);
+	xfr_via_socket(fl_filename_name(def_custom_transfer_filename.c_str()), fbuff);
+}
+
+void cb_save_custom_html(Fl_Widget *w, void *d)
+{
+	btn_save_custom_html_file->color(FL_BACKGROUND_COLOR);
+
+	if (receive_custom_buffer.empty()) return;
+
+	FILE *binfile = fopen(def_custom_rx_filename.c_str(), "wb");
+	if (binfile) {
+		fwrite(receive_custom_buffer.c_str(), receive_custom_buffer.length(), 1, binfile);
+		fclose(binfile);
+	}
+	txt_rcvd_custom_html_filename->value("");
+	update_custom_transfer();
+}
+
+void cb_btn_select_custom_html(Fl_Widget *w, void *d)
+{
+	def_custom_transfer_filename = custom_files[custom_selector->index()];
+	show_filename(def_custom_transfer_filename);
+	load_custom_html_file();
+}
