@@ -88,7 +88,7 @@ struct mg_server *server = (mg_server *)NULL;
 void *poll_server(void *);
 void start_web_server();
 void close_server();
-void get_next_port_number();
+bool get_next_port_number();
 
 int flmsg_webserver_portnbr = 8080;
 char flmsg_webserver_szportnbr[5];
@@ -126,13 +126,9 @@ const char *options[] = {\
 Fl_Double_Window *mainwindow = 0;
 Fl_Double_Window *optionswindow = 0;
 Fl_Double_Window *arlwindow = 0;
-Fl_Double_Window *config_files_window = 0;
-Fl_Double_Window *config_datetime_window = 0;
-Fl_Double_Window *config_personal_window = 0;
-Fl_Double_Window *config_radiogram_window = 0;
+Fl_Double_Window *config_dialog = 0;
 Fl_Double_Window *hxwindow = 0;
 Fl_Double_Window *header_window = 0;
-Fl_Double_Window *socket_window = 0;
 
 pthread_t *xmlrpc_thread = 0;
 pthread_mutex_t mutex_xmlrpc = PTHREAD_MUTEX_INITIALIZER;
@@ -197,7 +193,7 @@ bool check_mycall()
 	if (!progStatus.my_call.empty())
 		return true;
 
-	if (!config_personal_window->visible())
+	if (!config_dialog->visible())
 		cb_config_personal();
 
 	return false;
@@ -797,6 +793,7 @@ int eval_transfer_size()
 		case CSV:		return eval_csv_fsize();
 		case CUSTOM:	return eval_custom_fsize();
 		case TRANSFER:	return eval_transfer_fsize();
+		case CUSTOM_TRANSFER: return eval_transfer_custom_form_fsize();
 		case MARSDAILY:	return eval_mars_daily_fsize();
 		case MARSINEEI:	return eval_mars_ineei_fsize();
 		case MARSNET:	return eval_mars_net_fsize();
@@ -850,7 +847,7 @@ void cb_new()
 		case REDX5739: cb_redx_5739_new(); break;
 		case REDX5739A: cb_redx_5739A_new(); break;
 		case REDX5739B: cb_redx_5739B_new(); break;
-		default : ;
+		default : return;
 	}
 	clear_estimate();
 }
@@ -948,7 +945,8 @@ void wrap_import(const char *fname)
 	}
 
 	if (isok) {
-		if (inpbuffer.find("<transfer>") == string::npos)
+		if (!(inpbuffer.find("<transfer>") != string::npos &&
+			inpbuffer.find("<html_file>") != string::npos) )
 			remove_cr( inpbuffer );
 		if (inpbuffer.find("<flics") != string::npos ||
 			inpbuffer.find("<flmsg") != string::npos) {
@@ -1051,6 +1049,9 @@ void wrap_import(const char *fname)
 			} else if (inpbuffer.find("<transfer>") != string::npos) {
 				selected_form = TRANSFER;
 				cb_transfer_wrap_import(filename, inpbuffer);
+			} else if (inpbuffer.find("<html_form>") != string::npos) {
+				selected_form = CUSTOM_TRANSFER;
+				cb_custom_form_wrap_import(filename, inpbuffer);
 			} else if (!exit_after_print) {
 				selected_form = NONE;
 				if (!fl_choice2(_("Cannot identify file type\n\nOpen as text file?"), "yes", "no", NULL)) {
@@ -1197,6 +1198,7 @@ void cb_wrap_autosend()
 		case CSV: cb_csv_wrap_autosend(); break;
 		case CUSTOM: cb_custom_wrap_autosend(); break;
 		case TRANSFER: cb_transfer_wrap_autosend(); break;
+		case CUSTOM_TRANSFER: cb_transfer_custom_html(); break;
 		case MARSDAILY: cb_mars_daily_wrap_autosend(); break;
 		case MARSINEEI: cb_mars_ineei_wrap_autosend(); break;
 		case MARSNET: cb_mars_net_wrap_autosend(); break;
@@ -1209,7 +1211,7 @@ void cb_wrap_autosend()
 		case REDX5739: cb_redx_5739_wrap_autosend(); break;
 		case REDX5739A: cb_redx_5739A_wrap_autosend(); break;
 		case REDX5739B: cb_redx_5739B_wrap_autosend(); break;
-		default: ;
+		default: return;
 	}
 }
 
@@ -1248,7 +1250,7 @@ void cb_load_template()
 		case REDX5739: cb_redx_5739_load_template(); break;
 		case REDX5739A: cb_redx_5739A_load_template(); break;
 		case REDX5739B: cb_redx_5739B_load_template(); break;
-		default: ;
+		default: return;
 	}
 	estimate();
 }
@@ -1288,7 +1290,7 @@ void cb_save_template()
 		case REDX5739: cb_redx_5739_save_template(); break;
 		case REDX5739A: cb_redx_5739A_save_template(); break;
 		case REDX5739B: cb_redx_5739B_save_template(); break;
-		default: ;
+		default: return;
 	}
 }
 
@@ -1327,7 +1329,7 @@ void cb_save_as_template()
 		case REDX5739: cb_redx_5739_save_as_template(); break;
 		case REDX5739A: cb_redx_5739A_save_as_template(); break;
 		case REDX5739B: cb_redx_5739B_save_as_template(); break;
-		default: ;
+		default: return;
 	}
 }
 
@@ -1366,7 +1368,7 @@ void cb_open()
 		case REDX5739: cb_redx_5739_open(); break;
 		case REDX5739A: cb_redx_5739A_open(); break;
 		case REDX5739B: cb_redx_5739B_open(); break;
-		default : ;
+		default : return;
 	}
 	estimate();
 }
@@ -1406,7 +1408,7 @@ void cb_save_as()
 		case BLANK: cb_blank_save_as(); break;
 		case CSV: cb_csv_save_as(); break;
 		case CUSTOM: cb_custom_save_as(); break;
-		default: ;
+		default: return;
 	}
 }
 
@@ -1445,7 +1447,7 @@ void cb_save()
 		case BLANK: cb_blank_save(); break;
 		case CSV: cb_csv_save(); break;
 		case CUSTOM: cb_custom_save(); break;
-		default: ;
+		default: return;
 	}
 }
 
@@ -1484,7 +1486,7 @@ void cb_html()
 		case BLANK: cb_blank_html(); break;
 		case CSV: cb_csv_html(); break;
 		case CUSTOM: cb_custom_html(); break;
-		default: ;
+		default: return;
 	}
 }
 
@@ -1543,8 +1545,8 @@ void cb_text()
 		case REDX5739B: cb_redx_5739B_textout(); break;
 		case CSV: cb_csv_textout(); break;
 		case CUSTOM: cb_custom_textout(); break;
-		case BLANK:
-		default: cb_blank_textout();
+		case BLANK: cb_blank_textout(); break;
+		default : return;
 	}
 }
 
@@ -1684,6 +1686,8 @@ void show_filename(string p)
 		case TRANSFER:
 			base_transfer_filename = fl_filename_name(p.c_str());
 			break;
+		case CUSTOM_TRANSFER:
+			break;
 		default:
 			return;
 	}
@@ -1968,49 +1972,45 @@ int main(int argc, char *argv[])
 		showoptions();
 
 	{
-		string appname = argv[0];
+		size_t p;
 		string appdir;
 		char dirbuf[FL_PATH_MAX + 1];
-		fl_filename_expand(dirbuf, FL_PATH_MAX, appname.c_str());
+		fl_filename_expand(dirbuf, FL_PATH_MAX, argv[0]);
 		appdir.assign(dirbuf);
 
 #ifdef __WOE32__
-		size_t p = appdir.rfind("flmsg.exe");
-		if (p != std::string::npos) {
-			appdir.erase(p);
-			string testfile;
-			testfile.assign(appdir).append("NBEMS.DIR");
-			FILE *testdir = fopen(testfile.c_str(),"r");
-			if (testdir) {
-				fclose(testdir);
-				FLMSG_dir = appdir;
-				FLMSG_dir.append("NBEMS.files/");
-			}
-		}
-		if (FLMSG_dir.empty()) {
+		p = appdir.rfind("flmsg.exe");
+		appdir.erase(p);
+		p = appdir.find("FL_APPS/");
+		if (p != string::npos) {
+			FLMSG_dir.assign(appdir.substr(0, p + 8));
+		} else {
 			fl_filename_expand(dirbuf, FL_PATH_MAX, "$USERPROFILE/");
-			FLMSG_dir = dirbuf;
-			FLMSG_dir.append("NBEMS.files/"); // directory if appropriate
+			FLMSG_dir.assign(dirbuf);
 		}
+		FLMSG_dir.append("NBEMS.files/");
 #else
-		size_t p = appdir.rfind("flmsg");
-		if (p != std::string::npos) {
-			if (appdir.find("./flmsg") != std::string::npos) {
-				if (getcwd(dirbuf, FL_PATH_MAX))
-					appdir.assign(dirbuf).append("/");
-			} else
-				appdir.erase(p);
-			string testfile;
-			testfile.assign(appdir).append("NBEMS.DIR");
-			FILE *testdir = fopen(testfile.c_str(),"r");
-			if (testdir) {
-				fclose(testdir);
-				FLMSG_dir = appdir;
-			}
-		}
-		if (FLMSG_dir.empty()) {
+		fl_filename_absolute(dirbuf, FL_PATH_MAX, argv[0]);
+		appdir.assign(dirbuf);
+		p = appdir.rfind("flmsg");
+		if (p != string::npos)
+			appdir.erase(p);
+		p = appdir.find("FL_APPS/");
+		if (p != string::npos)
+			FLMSG_dir.assign(appdir.substr(0, p + 8));
+		else {
 			fl_filename_expand(dirbuf, FL_PATH_MAX, "$HOME/");
 			FLMSG_dir = dirbuf;
+		}
+
+		DIR *isdir = 0;
+		string test_dir;
+		test_dir.assign(FLMSG_dir).append("NBEMS.files/");
+		isdir = opendir(test_dir.c_str());
+		if (isdir) {
+			FLMSG_dir = test_dir;
+			closedir(isdir);
+		} else {
 			FLMSG_dir.append(".nbems/");
 		}
 #endif
@@ -2027,12 +2027,9 @@ int main(int argc, char *argv[])
 	Fl::add_handler(default_handler);
 #endif
 
-	config_files_window = config_files_dialog();
-	config_datetime_window = date_time_dialog();
-	config_personal_window = personal_dialog();
-	config_radiogram_window = radiogram_dialog();
+	config_dialog = create_config_dialog();
+
 	header_window = headers_dialog();
-	socket_window = socket_dialog();
 
 	btn_utc_format0->value(progStatus.UTC == 0);
 	btn_utc_format1->value(progStatus.UTC == 1);
@@ -2053,7 +2050,7 @@ int main(int argc, char *argv[])
 
 	checkdirectories();
 	load_custom_menu();
-	get_next_port_number();
+	if (!get_next_port_number()) return 1;
 
 	string debug_file = FLMSG_dir;
 	debug_file.append("debug_log_").append(flmsg_webserver_szportnbr).append(".txt");
@@ -2359,8 +2356,9 @@ void closeoptions()
 {
 	optionswindow->hide();
 }
+//extern Fl_Group	*tab_headers;
 
-void cb_config_date_time()
+void set_config_values()
 {
 	btn_dtformat0->value(progStatus.dtformat == 0);
 	btn_dtformat1->value(progStatus.dtformat == 1);
@@ -2373,32 +2371,17 @@ void cb_config_date_time()
 	btn_utc_format4->value(progStatus.UTC == 4);
 	btn_utc_format5->value(progStatus.UTC == 5);
 
-	config_datetime_window->show();
-}
-
-void cb_config_personal()
-{
 	txt_my_call->value(progStatus.my_call.c_str());
 	txt_my_name->value(progStatus.my_name.c_str());
 	txt_my_addr->value(progStatus.my_addr.c_str());
 	txt_my_city->value(progStatus.my_city.c_str());
 	txt_my_tel->value(progStatus.my_tel.c_str());
 
-	config_personal_window->show();
-}
-
-void cb_config_radiogram()
-{
 	cnt_wpl->value(progStatus.wpl);
 
 	txt_rgnbr->value(progStatus.rgnbr.c_str());
 	btn_rgnbr_fname->value(progStatus.rgnbr_fname);
 
-	config_radiogram_window->show();
-}
-
-void cb_config_files()
-{
 	btn_open_on_export->value(progStatus.open_on_export);
 	btn_use_compression->value(progStatus.use_compression);
 	txt_sernbr->value(progStatus.sernbr.c_str());
@@ -2408,14 +2391,46 @@ void cb_config_files()
 
 	txt_mars_roster_file->value(progStatus.mars_roster_file.c_str());
 
-	config_files_window->show();
+	txt_socket_addr->value(progStatus.socket_addr.c_str());
+	txt_socket_port->value(progStatus.socket_port.c_str());
+
+	txt_web_addr->value("127.0.0.1");
+	txt_web_port->value(flmsg_webserver_szportnbr);
+}
+
+void cb_config_date_time()
+{
+	set_config_values();
+	tabs_config->value(tab_date_time);
+	config_dialog->show();
+}
+
+void cb_config_personal()
+{
+	set_config_values();
+	tabs_config->value(tab_personal);
+	config_dialog->show();
+}
+
+void cb_config_radiogram()
+{
+	set_config_values();
+	tabs_config->value(tab_config_radiogram);
+	config_dialog->show();
+}
+
+void cb_config_files()
+{
+	set_config_values();
+	tabs_config->value(tab_files);
+	config_dialog->show();
 }
 
 void cb_config_socket()
 {
-	txt_socket_addr->value(progStatus.socket_addr.c_str());
-	txt_socket_port->value(progStatus.socket_port.c_str());
-	socket_window->show();
+	set_config_values();
+	tabs_config->value(tab_socket);
+	config_dialog->show();
 }
 
 void show_help()
@@ -2658,7 +2673,7 @@ void remove_port_number()
 	LOG_INFO("Closed Web server on localhost:%s", flmsg_webserver_szportnbr);
 }
 
-void get_next_port_number()
+bool get_next_port_number()
 {
 	string ports;
 	string chk_fname = FLMSG_dir;
@@ -2687,7 +2702,12 @@ void get_next_port_number()
 						sizeof(flmsg_webserver_szportnbr),
 						"%d", flmsg_webserver_portnbr);
 			}
-			ports.append(flmsg_webserver_szportnbr).append("\n");
+			if (flmsg_webserver_portnbr <= 8130)
+				ports.append(flmsg_webserver_szportnbr).append("\n");
+			else {
+				fl_alert2("Exceeded max web port 8130");
+				return false;
+			}
 		}
 		ofstream wchkfile(chk_fname.c_str());
 		if (wchkfile) {
@@ -2704,6 +2724,7 @@ void get_next_port_number()
 			wchkfile.close();
 		}
 	}
+	return true;
 }
 
 
@@ -2729,9 +2750,9 @@ void * poll_server(void *d)
 		n--;
 		if (!n) {
 			Fl::awake(load_custom, 0);
-			n = 10;
+			n = 100;
 		}
-		MilliSleep(100);
+		MilliSleep(10);
 	}
 	return NULL;
 }
@@ -2751,11 +2772,13 @@ static int web_handler(struct mg_connection *conn)
 	if (strcmp(conn->uri, "/handle_post_request") == 0) {
 		get_html_vars(conn);
 		custom_viewer(conn);
-  } else {
-		if (handle_type == HANDLE_EDIT)
+	} else {
+		if (handle_type == HANDLE_EDIT) {
 			custom_editor(conn);
-		else if (handle_type == HANDLE_VIEW)
+		}
+		else if (handle_type == HANDLE_VIEW) {
 			custom_viewer(conn);
+		}
 		else if (handle_type == HANDLE_WAITING)
 // Show HTML waiting
 			mg_send_data(conn, html_waiting, strlen(html_waiting));
